(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define([], factory);
  } else if (typeof exports !== "undefined") {
    factory();
  } else {
    var mod = {
      exports: {}
    };
    factory();
    global.didParser = mod.exports;
  }
})(this, function () {
  "use strict";

  var didParser = function didParser(did) {
    if (!did) throw new Error("DID is not provided: ".concat(did));
    var parts = did.match(/^(did)\:([a-zA-Z0-9]+)\:([a-zA-Z0-9]+)$/);
    if (!parts || parts.length !== 4) throw new Error("DID is not recognized: ".concat(JSON.stringify(parts)));
    var parsed = {
      method: parts[2],
      id: parts[3]
    };

    switch (parsed.method) {
      case 'eth':
      case 'ethr':
      case 'uport':
      case 'muport':
      case '7hirten':
      case 'spherity':
        return parsed;

      default:
        throw new Error("Unsupported DID method: ".concat(parsed.method));
    }
  };

  module.exports = didParser;
});