(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["@babel/runtime/regenerator", "@babel/runtime/helpers/asyncToGenerator"], factory);
  } else if (typeof exports !== "undefined") {
    factory(require("@babel/runtime/regenerator"), require("@babel/runtime/helpers/asyncToGenerator"));
  } else {
    var mod = {
      exports: {}
    };
    factory(global.regenerator, global.asyncToGenerator);
    global.resolve = mod.exports;
  }
})(this, function (_regenerator, _asyncToGenerator2) {
  "use strict";

  var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

  _regenerator = _interopRequireDefault(_regenerator);
  _asyncToGenerator2 = _interopRequireDefault(_asyncToGenerator2);

  var didParser = require('./didParser');

  var didResolver = require('./didResolver');

  var didPattern = /^(did)\:([a-zA-Z0-9]+)\:([a-zA-Z0-9]{42})$/;

  var filterPublicKey = function filterPublicKey(doc, fragment) {
    if (!doc) throw new Error('Document is not provided');
    if (!doc.publicKey) return new Error('Public key array is not found');
    if (!fragment) throw new Error('Fragment is not provided');
    return doc.publicKey.find(function (pubKey) {
      return pubKey.id === "".concat(doc.id).concat(fragment);
    });
  };

  var filterDelegates = function filterDelegates(doc) {
    if (!doc) throw new Error('Document is not provided');
    if (!doc.publicKey) return new Error('Public key array is not found');
    return doc.publicKey.filter(function (x) {
      return x.id.includes('delegate');
    });
  };

  var filterOwnerAddress = function filterOwnerAddress(doc) {
    var isSEC = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    var prefix = isSEC ? 'sec:' : ''; // TODO: remove after solving the issue

    if (!doc) throw new Error('Document is not provided');
    var publicKey = filterPublicKey(doc, '#owner');
    if (!publicKey) throw new Error('Public key is not found');
    if (!(publicKey.ethereumAddress || publicKey["".concat(prefix, "ethereumAddress")])) throw new Error('Owner address is not found');
    return publicKey.ethereumAddress || publicKey["".concat(prefix, "ethereumAddress")];
  };

  var filterService = function filterService(doc, serviceType) {
    if (!doc) throw new Error('Document is not provided');
    if (!serviceType) throw new Error('Service type is not provided');
    if (!doc.service) return undefined;
    return doc.service.find(function (service) {
      return service.type === serviceType;
    });
  };

  var filterAgentServiceUrl = function filterAgentServiceUrl(doc) {
    if (!doc) throw new Error('Document is not provided');
    var service = filterService(doc, 'AgentService');
    if (!service) return undefined;
    return service.serviceEndpoint || undefined;
  };

  var resolve =
  /*#__PURE__*/
  function () {
    var _ref = (0, _asyncToGenerator2["default"])(
    /*#__PURE__*/
    _regenerator["default"].mark(function _callee(resolver, did) {
      var isSEC,
          _didParser,
          method,
          doc,
          delegates,
          ownerAddress,
          agentServiceUrl,
          _args = arguments;

      return _regenerator["default"].wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              isSEC = _args.length > 2 && _args[2] !== undefined ? _args[2] : true;

              if (didPattern.test(did)) {
                _context.next = 3;
                break;
              }

              throw new Error("Invalid DID: ".concat(did));

            case 3:
              _didParser = didParser(did), method = _didParser.method;
              _context.next = 6;
              return didResolver(resolver || 'UniResolverSpherity')(did);

            case 6:
              doc = _context.sent;
              delegates = filterDelegates(doc);
              ownerAddress = filterOwnerAddress(doc, isSEC);
              agentServiceUrl = filterAgentServiceUrl(doc);
              return _context.abrupt("return", {
                doc: doc,
                owner: "did:".concat(method, ":").concat(ownerAddress),
                delegates: delegates,
                agentServiceUrl: agentServiceUrl
              });

            case 11:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function resolve(_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }();

  module.exports = {
    resolve: resolve,
    filterPublicKey: filterPublicKey,
    filterDelegates: filterDelegates,
    filterOwnerAddress: filterOwnerAddress,
    filterService: filterService,
    filterAgentServiceUrl: filterAgentServiceUrl
  };
});